package com.aitrainer.api.controller

import com.aitrainer.api.model.Exercises
import com.aitrainer.api.repository.ExercisesRepository
import org.springframework.http.HttpStatus
import org.springframework.http.ResponseEntity
import org.springframework.web.bind.annotation.*
import javax.validation.Valid

@RestController
@RequestMapping("/api")
class ExerciseController(private val exercisesRepository: ExercisesRepository) {

    @GetMapping("/exercises/{id}")
    fun getExerciseById(@PathVariable(value = "id") exerciseId: Long): ResponseEntity<Exercises> {
        return exercisesRepository.findById(exerciseId).map { exercise ->
            ResponseEntity.ok(exercise)
        }.orElse(ResponseEntity.notFound().build())
    }

    @GetMapping("/exercises/customer/{id}")
    fun getAllExercisesByCustomerId(@PathVariable( value = "id" ) customerId: Long? ): List<Exercises> =
            exercisesRepository.getAllByCustomerIdOrderByDateAddDesc(customerId)

    @GetMapping("/exercises/exercise_type/{id}")
    fun getAllExercisesByExerciseTypeId(@PathVariable( value = "id" ) exerciseTypeId: Long ): ResponseEntity<List<Exercises>> {
        val exercises:List<Exercises> = exercisesRepository.getAllByExerciseTypeId(exerciseTypeId)
        return if (exercises.isEmpty()) ResponseEntity.notFound().build() else
            ResponseEntity.ok().body(exercises)
    }


    @PostMapping("/exercises")
    fun createNewExercise(@Valid @RequestBody exercise: Exercises): ResponseEntity<Exercises> {
        return ResponseEntity.ok().body(exercisesRepository.save(exercise))
    }

    @PutMapping("/exercises/{id}")
    fun updateExerciseById(@PathVariable(value = "id") exerciseId: Long,
                           @Valid @RequestBody newExercises: Exercises): ResponseEntity<Exercises> {
        return exercisesRepository.findById(exerciseId).map { existingExercises ->
            val updatedExercises: Exercises = existingExercises.copy(
                    exerciseTypeId = newExercises.exerciseTypeId,
                    customerId = newExercises.customerId,
                    dateAdd = newExercises.dateAdd,
                    quantity = newExercises.quantity,
                    restTime = newExercises.restTime
            )
            ResponseEntity.ok().body(exercisesRepository.save(updatedExercises))
        }.orElse(ResponseEntity.notFound().build())
    }


    @PostMapping("/exercises/{id}")
    fun deleteExerciseById(@PathVariable(value = "id") exerciseId: Long): HttpStatus {
        val exercise2Delete = exercisesRepository.findById(exerciseId).orElse(null)
        exercisesRepository.delete(exercise2Delete)
        return HttpStatus.OK
    }
}