package com.aitrainer.api.test

import com.aitrainer.api.controller.CustomerPackageController
import com.aitrainer.api.model.*
import com.aitrainer.api.model.diet.*
import com.aitrainer.api.repository.*
import com.aitrainer.api.repository.diet.*
import com.google.gson.Gson
import com.google.gson.reflect.TypeToken
import org.junit.jupiter.api.Test
import org.junit.jupiter.api.TestInstance
import org.springframework.beans.factory.annotation.Autowired
import org.springframework.boot.test.context.SpringBootTest
import org.springframework.http.HttpStatus
import org.springframework.http.ResponseEntity
import kotlin.test.assertEquals
import kotlin.test.assertTrue

@SpringBootTest
@TestInstance(TestInstance.Lifecycle.PER_CLASS)
class AppCustomerPackageTest {

    @Autowired
    private lateinit var customerRepository: CustomerRepository

    @Autowired
    private lateinit var customerExerciseDeviceRepository: CustomerExerciseDeviceRepository

    @Autowired
    private lateinit var exercisesRepository: ExercisesRepository

    @Autowired
    private lateinit var purchaseRepository: PurchaseRepository

    @Autowired
    private lateinit var customerPropertyRepository: CustomerPropertyRepository

    @Autowired
    private lateinit var exerciseResultRepository: ExerciseResultRepository

    @Autowired
    private lateinit var customerActivityRepository: CustomerActivityRepository

    @Autowired
    private lateinit var customerTrainingPlanRepository: CustomerTrainingPlanRepository

    @Autowired
    private lateinit var customerMembershipRepository: CustomerMembershipRepository

    @Autowired
    private lateinit var  dietRepository: DietRepository

    @Autowired
    private lateinit var  dietUserConsumptionRepository: DietUserConsumptionRepository

    @Autowired
    private lateinit var  dietUserRepository: DietUserRepository

    @Autowired
    private lateinit var  dietUserPreferenceRepository: DietUserPreferenceRepository

    @Autowired
    private lateinit var  dietUserSensitivityRepository: DietUserSensitivityRepository

    @Autowired
    private lateinit var customerConversationRepository: CustomerConversationRepository

    @Test
    fun dietCustomerPackageTest() {
        val gson = Gson()

        val controller = CustomerPackageController(
            customerRepository,
            customerExerciseDeviceRepository,
            exercisesRepository,
            purchaseRepository,
            customerPropertyRepository,
            exerciseResultRepository,
            customerActivityRepository,
            customerTrainingPlanRepository,
            customerMembershipRepository,
            dietRepository,
            dietUserConsumptionRepository,
            dietUserRepository,
            dietUserPreferenceRepository,
            dietUserSensitivityRepository,
            customerConversationRepository
        )

        val response = controller.getDietCustomerPackageData(2)
        assertEquals(response.statusCode, HttpStatus.OK)

        val dietCustomerPackageJson: String = response.body as String
        assertTrue(dietCustomerPackageJson.isNotEmpty())

        val packages = dietCustomerPackageJson.split("|||").toTypedArray()

        packages.forEach {
            val record = it.split("***")
            print(record[0] + "\n")
            when {
                record[0] == Diet::class.simpleName -> {
                    val dietJson: String = record[1]
                    val type = object : TypeToken<List<Diet?>?>() {}.type
                    val list: List<Diet> = gson.fromJson(dietJson, type)
                    assertEquals(list[0].dietText, "Test diet text")
                    assertEquals(list[0].meals[0].mealName, "monday|breakfast")
                    assertEquals(list[0].meals[0].quantity, 2.0)
                    assertEquals(list[0].meals[2].quantityUnit, "tányér")
                }

                record[0] == DietUserConsumption::class.simpleName -> {
                    val dietUserConsumptionJson: String = record[1]
                    val type = object : TypeToken<List<DietUserConsumption?>?>() {}.type
                    val list: List<DietUserConsumption> = gson.fromJson(dietUserConsumptionJson, type)
                    assertEquals(list[0].name, "Tükörtojás")
                    assertEquals(list[0].quantity, 120.0)
                    assertEquals(list[0].ch, 1.0)
                }

                record[0] == DietUser::class.simpleName -> {
                    val dietUserJson: String = record[1]
                    val type = object : TypeToken<DietUser?>() {}.type
                    val list: DietUser = gson.fromJson(dietUserJson, type)
                    assertEquals(list.customerId, 764)
                }

                record[0] == DietUserPreference::class.simpleName -> {
                    val dietUserPreferenceJson: String = record[1]
                    val type = object : TypeToken<List<DietUserPreference?>?>() {}.type
                    val list: List<DietUserPreference> = gson.fromJson(dietUserPreferenceJson, type)
                    assertEquals(list[0].dietUserId, 2)
                    assertEquals(list[0].temperature, -1)
                }

                record[0] == DietUserSensitivity::class.simpleName -> {
                    val dietUserSensitivityJson: String = record[1]
                    val type = object : TypeToken<List<DietUserSensitivity?>?>() {}.type
                    val list: List<DietUserSensitivity> = gson.fromJson(dietUserSensitivityJson, type)
                    assertEquals(list[0].dietUserId, 2)
                    assertEquals(list[0].sensitivityId, 1)
                }
            }
        }
    }

    @Test
    fun customerClubPackageTest() {
        val gson = Gson()

        val controller = CustomerPackageController(
            customerRepository,
            customerExerciseDeviceRepository,
            exercisesRepository,
            purchaseRepository,
            customerPropertyRepository,
            exerciseResultRepository,
            customerActivityRepository,
            customerTrainingPlanRepository,
            customerMembershipRepository,
            dietRepository,
            dietUserConsumptionRepository,
            dietUserRepository,
            dietUserPreferenceRepository,
            dietUserSensitivityRepository,
            customerConversationRepository
        )

        val response = controller.getCustomerClubPackageData(90)
        assertEquals(response.statusCode, HttpStatus.OK)

        val clubPackageJson: String = response.body as String
        assertTrue(clubPackageJson.isNotEmpty())


        val packages = clubPackageJson.split("|||").toTypedArray()

        packages.forEach {
            val record = it.split("***")
            print(record[0] + "\n")
            when {
                record[0] == CustomerMembership::class.simpleName -> {
                    val customerMembershipJson: String = record[1]
                    val type = object : TypeToken<List<CustomerMembership?>?>() {}.type
                    val customerMembershipList: List<CustomerMembership> = gson.fromJson(customerMembershipJson, type)
                    assertEquals(customerMembershipList.size, 2)
                    assertEquals(customerMembershipList[0].membershipId, 1)
                }

                record[0] == CustomerProperty::class.simpleName -> {
                    //actual values
                    val propertyJson: String = record[1]
                    val type = object : TypeToken<List<CustomerProperty?>?>() {}.type
                    val propertyList: List<CustomerProperty> = gson.fromJson(propertyJson, type)
                    assertTrue(propertyList.isNotEmpty())
                    print ("list size: ${propertyList.size}")
                    assertEquals(propertyList.size, 6)
                    assertEquals(propertyList[5].propertyId,1)
                    assertEquals(propertyList[5].propertyValue,75.0)
                    assertEquals(propertyList[5].goal,true)
                }

            }
        }
    }

    @OptIn(ExperimentalStdlibApi::class)
    @Test
    fun customerPackageTest() {
        val gson = Gson()

        val controller = CustomerPackageController(customerRepository, customerExerciseDeviceRepository,
            exercisesRepository, purchaseRepository, customerPropertyRepository,
            exerciseResultRepository, customerActivityRepository, customerTrainingPlanRepository,
            customerMembershipRepository,
            dietRepository,
            dietUserConsumptionRepository,
            dietUserRepository,
            dietUserPreferenceRepository,
            dietUserSensitivityRepository,
            customerConversationRepository)

        var response: ResponseEntity<*> = controller.getCustomerPackageData(91)
        assertEquals(response.statusCode, HttpStatus.NOT_FOUND)

        response = controller.getCustomerPackageData(90)
        assertEquals(response.statusCode, HttpStatus.OK)

        val appPackageJson: String = response.body as String
        assertTrue(appPackageJson.isNotEmpty())


        val packages = appPackageJson.split("|||").toTypedArray()

        packages.forEach {
            val record = it.split("***")
            print(record[0] + "\n")
            when {
                record[0] == Customer::class.simpleName -> {
                    val customerJson: String = record[1]
                    val type = object : TypeToken<Customer?>(){}.type
                    val customer: Customer = gson.fromJson(customerJson, type)
                    assertEquals(customer.email, "sw@andio.biz")
                    assertEquals(customer.birthYear, 1972)
                    assertEquals(customer.trialDate, "2021-08-07 08:00:00")
                    assertEquals(customer.syncedDate, "2021-08-06 08:00:00")
                    assertEquals(customer.firebaseRegToken, "123456789asdfghjkl")
                    assertEquals(customer.phone, "123456")
                    assertEquals(customer.lifeLong, 1)
                }
                record[0] == Exercises::class.simpleName -> {
                    val exercisesJson: String = record[1]
                    val type = object : TypeToken<List<Exercises?>?>() {}.type
                    val exerciseList: List<Exercises> = gson.fromJson(exercisesJson, type)
                    assertTrue(exerciseList.isNotEmpty())
                    assertEquals(exerciseList[0].exerciseTypeId, 2)
                    assertEquals(exerciseList[0].quantity, 34.0)
                }
                record[0] == CustomerExerciseDevice::class.simpleName -> {
                    val exerciseDeviceJson: String = record[1]
                    val type = object : TypeToken<List<CustomerExerciseDevice?>?>() {}.type
                    val customerExerciseDeviceList: List<CustomerExerciseDevice> = gson.fromJson(exerciseDeviceJson, type)
                    assertTrue(customerExerciseDeviceList.isNotEmpty())
                    assertEquals(customerExerciseDeviceList[1].exerciseDeviceId,2)
                }

                record[0] == CustomerProperty::class.simpleName -> {
                    //actual values
                    val propertyJson: String = record[1]
                    val type = object : TypeToken<List<CustomerProperty?>?>() {}.type
                    val propertyList: List<CustomerProperty> = gson.fromJson(propertyJson, type)
                    assertTrue(propertyList.isNotEmpty())
                    assertEquals(propertyList[2].propertyId,5)
                    assertEquals(propertyList[2].propertyValue,42.0)
                }
                record[0] == CustomerProperty::class.simpleName+"All" -> {
                    val propertyJson: String = record[1]
                    val type = object : TypeToken<List<CustomerProperty?>?>() {}.type
                    val propertyList: List<CustomerProperty> = gson.fromJson(propertyJson, type)
                    assertTrue(propertyList.isNotEmpty())
                    assertEquals(propertyList[1].propertyId,1)
                    assertEquals(propertyList[1].propertyValue,82.0)
                }
                record[0] == CustomerActivity::class.simpleName+"All" -> {
                    val activityJson: String = record[1]
                    val type = object : TypeToken<List<CustomerActivity?>?>() {}.type
                    val activityList: List<CustomerActivity> = gson.fromJson(activityJson, type)
                    assertTrue(activityList.isNotEmpty())
                    assertEquals(activityList[0].type,"basic_tutorial")
                }
                record[0] == CustomerTrainingPlan::class.simpleName+"All" -> {
                    val trainingPlanJson: String = record[1]
                    val type = object : TypeToken<List<CustomerTrainingPlan?>?>() {}.type
                    val trainingPlanList: List<CustomerTrainingPlan> = gson.fromJson(trainingPlanJson, type)
                    assertTrue(trainingPlanList.isNotEmpty())
                    assertEquals(trainingPlanList[0].customerId,91)
                    assertEquals(trainingPlanList[0].trainingPlanId,2)
                    assertEquals(trainingPlanList[0].details.size,2)
                    assertEquals(trainingPlanList[0].details[0].exerciseTypeId,60)
                    assertEquals(trainingPlanList[0].details[0].weight,24.0)
                }
            }
        }

        // deactivated customer
        response = controller.getCustomerPackageData(1)
        assertEquals(response.statusCode, HttpStatus.NOT_FOUND)

    }

}