package com.aitrainer.api.test

import com.aitrainer.api.controller.PackageController
import com.aitrainer.api.model.*
import com.aitrainer.api.model.diet.DietSensitivity
import com.aitrainer.api.model.diet.Meal
import com.aitrainer.api.model.diet.Recipe
import com.aitrainer.api.model.diet.Store
import com.aitrainer.api.repository.*
import com.aitrainer.api.repository.diet.*
import org.junit.jupiter.api.Test
import org.junit.jupiter.api.TestInstance
import org.springframework.beans.factory.annotation.Autowired
import org.springframework.boot.test.context.SpringBootTest
import org.springframework.http.ResponseEntity
import com.google.gson.Gson
import com.google.gson.reflect.TypeToken
import kotlin.test.assertEquals
import kotlin.test.assertTrue

@SpringBootTest
@TestInstance(TestInstance.Lifecycle.PER_CLASS)
class AppPackageTest {

    @Autowired
    private lateinit var exerciseAbilityRepository: ExerciseAbilityRepository
    @Autowired
    private lateinit var exerciseTypeRepository: ExerciseTypeRepository
    @Autowired
    private lateinit var  exerciseTreeRepository: ExerciseTreeRepository
    @Autowired
    private lateinit var  propertyRepository: PropertyRepository
    @Autowired
    private lateinit var  productRepository: ProductRepository
    @Autowired
    private lateinit var  exerciseDeviceRepository: ExerciseDeviceRepository
    @Autowired
    private lateinit var exerciseTreeParentsRepository: ExerciseTreeParentsRepository
    @Autowired
    private lateinit var exercisePlanTemplateRepository: ExercisePlanTemplateRepository
    @Autowired
    private lateinit var evaluationRepository: EvaluationRepository
    @Autowired
    private lateinit var sportRepository: SportRepository
    @Autowired
    private lateinit var tutorialRepository: TutorialRepository
    @Autowired
    private lateinit var descriptionRepository: DescriptionRepository
    @Autowired
    private lateinit var faqRepository: FaqRepository
    @Autowired
    private lateinit var trainingPlanRepository: TrainingPlanRepository
    @Autowired
    private lateinit var splitTestsRepository: SplitTestsRepository
    @Autowired
    private lateinit var trainingPlanDayRepository: TrainingPlanDayRepository
    @Autowired
    private lateinit var appTextRepository: AppTextRepository
    @Autowired
    private lateinit var trainingProgramRepository: TrainingProgramRepository
    @Autowired
    private lateinit var membershipRepository: MembershipRepository

    @Autowired
    private lateinit var dietSensitivityRepository: DietSensitivityRepository

    @Autowired
    private lateinit var mealRepository: MealRepository

    @Autowired
    private lateinit var recipeRepository: RecipeRepository

    @Autowired
    private lateinit var storeRepository: StoreRepository


    @Test
    fun testDietPackage() {
        val gson = Gson()

        val controller = PackageController(
            exerciseAbilityRepository,
            exerciseTypeRepository,
            exerciseTreeRepository,
            propertyRepository,
            productRepository,
            exerciseDeviceRepository,
            exerciseTreeParentsRepository,
            exercisePlanTemplateRepository,
            evaluationRepository,
            sportRepository,
            tutorialRepository,
            descriptionRepository,
            faqRepository,
            trainingPlanRepository,
            splitTestsRepository,
            trainingPlanDayRepository,
            appTextRepository,
            trainingProgramRepository,
            membershipRepository,
            storeRepository,
            recipeRepository,
            mealRepository,
            dietSensitivityRepository,
        )
        val response: ResponseEntity<*> = controller.getDietPackageData()

        val dietPackageJson: String = response.body as String
        assertTrue(dietPackageJson.isNotEmpty())


        val packages = dietPackageJson.split("|||").toTypedArray()

        var appText = false
        packages.forEach {
            val record = it.split("***")
            if (record[0] == Membership::class.simpleName) {
                val membershipJson: String = record[1]
                val type = object : TypeToken<List<Membership?>?>() {}.type
                val listMembership: List<Membership> = gson.fromJson(membershipJson, type)
                assertTrue(listMembership.isNotEmpty())
            } else if ( record[0] == Store::class.simpleName) {
                val storeJson: String = record[1]
                val type = object : TypeToken<List<Store?>?>() {}.type
                val listStore: List<Store> = gson.fromJson(storeJson, type)
                assertTrue(listStore.isNotEmpty())
                assertEquals(listStore[0].storeName, "Tesco")
                assertEquals(listStore[0].country, "HU")
                assertEquals(listStore[3].country, "DE")
                assertEquals(listStore[3].storeName, "Aldi")

            } else if ( record[0] == Recipe::class.simpleName) {
                val recipeJson: String = record[1]
                val type = object : TypeToken<List<Recipe?>?>() {}.type
                val list: List<Recipe> = gson.fromJson(recipeJson, type)
                assertTrue(list.isNotEmpty())
                assertEquals(list[0].name, "Bableves")
                assertEquals(list[0].cal, 180)
                assertEquals(list[0].dietUserId, 4)
                assertEquals(list[2].name, "Borsóleves")
                assertEquals(list[2].protein, 45.0)
                assertEquals(list[2].dietUserId, 1)
            } else if ( record[0] == Meal::class.simpleName) {
                val mealJson: String = record[1]
                val type = object : TypeToken<List<Meal?>?>() {}.type
                val list: List<Meal> = gson.fromJson(mealJson, type)
                assertTrue(list.isNotEmpty())
                assertEquals(list[0].name, "Tükörtojás1")
                assertEquals(list[0].calMin, 70.0)
                assertEquals(list[0].proteinMax, 25.0)
                assertEquals(list[1].name, "Tigris buci1")
            } else if ( record[0] == DietSensitivity::class.simpleName) {
                val dietSensitivityJson: String = record[1]
                val type = object : TypeToken<List<DietSensitivity?>?>() {}.type
                val list: List<DietSensitivity> = gson.fromJson(dietSensitivityJson, type)
                assertTrue(list.isNotEmpty())
                assertEquals(list[0].name, "Nut Allergy")
                assertEquals(list[1].name, "Penicillin sensitivity")
            } else if ( record[0] == AppText::class.simpleName) {
                val appTextJson: String = record[1]
                val type = object : TypeToken<List<AppText?>?>() {}.type
                val list: List<AppText> = gson.fromJson(appTextJson, type)
                assertTrue(list.isNotEmpty())
                assertEquals(list[0].textKey, "monday")
                assertEquals(list[1].translations[0].translation, "kedd")
                assertEquals(list[6].textKey, "sunday")
                appText = true

            }
        }

        assertEquals(appText, true)
    }

    @Test
    fun testClubPackage() {
        val gson = Gson()

        val controller = PackageController(
            exerciseAbilityRepository,
            exerciseTypeRepository,
            exerciseTreeRepository,
            propertyRepository,
            productRepository,
            exerciseDeviceRepository,
            exerciseTreeParentsRepository,
            exercisePlanTemplateRepository,
            evaluationRepository,
            sportRepository,
            tutorialRepository,
            descriptionRepository,
            faqRepository,
            trainingPlanRepository,
            splitTestsRepository,
            trainingPlanDayRepository,
            appTextRepository,
            trainingProgramRepository,
            membershipRepository,
            storeRepository,
            recipeRepository,
            mealRepository,
            dietSensitivityRepository,
        )
        val response: ResponseEntity<*> = controller.getClubPackageData()


        val clubPackageJson: String = response.body as String
        assertTrue(clubPackageJson.isNotEmpty())


        val packages = clubPackageJson.split("|||").toTypedArray()

        packages.forEach {
            val record = it.split("***")
            if (record[0] == Membership::class.simpleName) {
                val membershipJson: String = record[1]
                val type = object : TypeToken<List<Membership?>?>() {}.type
                val listMembership: List<Membership> = gson.fromJson(membershipJson, type)
                assertTrue(listMembership.isNotEmpty())
                assertEquals(listMembership[1].name, "Lifetime")
                assertEquals(listMembership[3].durationUnit, "month")
                assertEquals(listMembership[2].durationType, "limited")

                assertEquals(listMembership.size, 5)
                assertEquals(listMembership[0].name, "Free")
                assertEquals(listMembership[4].name, "Mezomorph Free 1W")
                assertEquals(listMembership[4].duration, 1)
                assertEquals(listMembership[4].durationUnit, "week")
                assertEquals(listMembership[4].trainingPlanId, 99)
                assertEquals(listMembership[4].trainingPlanDayIds, "21|22")
            }
        }
    }

    @Test
    fun testAppPackage() {
        val gson = Gson()

        val controller = PackageController(
            exerciseAbilityRepository,
            exerciseTypeRepository,
            exerciseTreeRepository,
            propertyRepository,
            productRepository,
            exerciseDeviceRepository,
            exerciseTreeParentsRepository,
            exercisePlanTemplateRepository,
            evaluationRepository,
            sportRepository,
            tutorialRepository,
            descriptionRepository,
            faqRepository,
            trainingPlanRepository,
            splitTestsRepository,
            trainingPlanDayRepository,
            appTextRepository,
            trainingProgramRepository,
            membershipRepository,
            storeRepository,
            recipeRepository,
            mealRepository,
            dietSensitivityRepository,
        )
        val response: ResponseEntity<*> = controller.getPackageData()

        val appPackageJson: String = response.body as String
        assertTrue(appPackageJson.isNotEmpty())


        val packages = appPackageJson.split("|||").toTypedArray()

        packages.forEach {
            val record = it.split("***")
            print(record[0] + "\n")
            if (record[0] == ExerciseType::class.simpleName) {
                //print("List ExerciseType: " + record[1])
                val exerciseTypeJson: String = record[1]
                val type = object : TypeToken<List<ExerciseType?>?>() {}.type
                val listExerciseType: List<ExerciseType> = gson.fromJson(exerciseTypeJson, type)
                assertTrue(listExerciseType.isNotEmpty())
                assertEquals(listExerciseType[1].name, "Húzódzkodás")
                assertEquals(listExerciseType[1].buddyWarning, true)
                //assertEquals(listExerciseType[1].translations[0].name, "Bicepsz")
            } else if (record[0] == ExerciseTree::class.simpleName) {
                val exerciseTreeJson: String = record[1]
                val type = object : TypeToken<List<ExerciseTree?>?>() {}.type
                val listExerciseTree: List<ExerciseTree> = gson.fromJson(exerciseTreeJson, type)
                assertTrue(listExerciseTree.isNotEmpty())
                assertEquals(listExerciseTree[1].name, "Muscle Build / Shape Toning")
                assertEquals(listExerciseTree[1].internalName, "one_rep_max")
                assertEquals(listExerciseTree[1].translations[0].name, "Izomépítés / Alakformálás")
            } else if (record[0] == Property::class.simpleName) {
                val propertyJson: String = record[1]
                val type = object : TypeToken<List<Property?>?>() {}.type
                val listProperty: List<Property> = gson.fromJson(propertyJson, type)
                assertTrue(listProperty.isNotEmpty())
                assertEquals(listProperty[2].propertyName, "Chest")
                assertEquals(listProperty[0].translations[0].propertyName, "Tömeg")
            } else if (record[0] == ExerciseTreeParents::class.simpleName) {
                val exerciseTreeParentsJson: String = record[1]
                val type = object : TypeToken<List<ExerciseTreeParents?>?>() {}.type
                val listParents: List<ExerciseTreeParents> = gson.fromJson(exerciseTreeParentsJson, type)
                assertTrue(listParents.isNotEmpty())
                assertEquals(listParents[2].exerciseTreeParentId, 4)
                assertEquals(listParents[6].exerciseTreeParentId, 4)
                assertEquals(listParents[6].exerciseTreeChildId, 17)
            } else if (record[0] == ExercisePlanTemplate::class.simpleName) {
                val exercisePlanTemplateJson: String = record[1]
                val type = object : TypeToken<List<ExercisePlanTemplate?>?>() {}.type
                val listTemplate: List<ExercisePlanTemplate> = gson.fromJson(exercisePlanTemplateJson, type)
                assertTrue(listTemplate.isNotEmpty())
                assertEquals(listTemplate[0].name, "Own Body")
                assertEquals(listTemplate[0].templateType, "mini_test_set")
                assertEquals(listTemplate[0].translations[0].name, "Saját testes tesztek")
                assertEquals(listTemplate[0].details[0].exerciseTypeId, 2)
                assertEquals(listTemplate[1].details[1].exerciseTypeId, 33)
                assertEquals(listTemplate[1].details[1].sort, 2)
            } else if (record[0] == Evaluation::class.simpleName) {
                val evaluationJson: String = record[1]
                val type = object : TypeToken<List<Evaluation?>?>() {}.type
                val evaluations: List<Evaluation> = gson.fromJson(evaluationJson, type)
                assertTrue(evaluations.isNotEmpty())
                assertEquals(evaluations.size, 2)
                assertEquals(evaluations[0].name, "PushUps")
                assertEquals(evaluations[0].attributes.size, 18)
                assertEquals(evaluations[0].attributes[1].name, "Fekvőtámasz_ffi_17-19_fair")
            } else if (record[0] == Sport::class.simpleName) {
                val sportJson: String = record[1]
                val type = object : TypeToken<List<Sport?>?>() {}.type
                val sports: List<Sport> = gson.fromJson(sportJson, type)
                assertEquals(sports.size, 5)
                assertEquals(sports[1].name, "Football")
                assertEquals(sports[1].translations[0].sportName, "Labdarúgás")
                assertEquals(sports[2].name, "Footgolf")
                assertEquals(sports[2].translations[0].sportName, "Footgolf")
                assertEquals(sports[3].translations[0].sportName, "Kézilabda")
            } else if (record[0] == Tutorial::class.simpleName) {
                val tutorialJson: String = record[1]
                val type = object : TypeToken<List<Tutorial?>?>() {}.type
                val tutorials: List<Tutorial> = gson.fromJson(tutorialJson, type)
                assertEquals(tutorials.size, 2)
                assertEquals(tutorials[0].name, "Basic")
                assertEquals(tutorials[0].steps.size, 9)
                assertEquals(
                    tutorials[0].steps[2].condition,
                    "{\"direction\":\"down\",\"top\":95,\"left\":-1,\"show_bubble\":false,\"bubble_x\":220,\"bubble_y\":220,\"bubble_width\":440,\"bubble_height\":440,\"show_check_text\":false,\"parent\":2}"
                )
                assertEquals(tutorials[0].steps[0].translations.size, 1)
            } else if (record[0] == Description::class.simpleName) {
                val descriptionJson: String = record[1]
                val type = object : TypeToken<List<Description?>?>() {}.type
                val descriptions: List<Description> = gson.fromJson(descriptionJson, type)
                assertEquals(descriptions.size, 2)
                assertEquals(descriptions[0].translations.size, 1)
                assertEquals(descriptions[0].name, "sales_page")
            } else if (record[0] == Faq::class.simpleName) {
                val faqJson: String = record[1]
                val type = object : TypeToken<List<Faq?>?>() {}.type
                val faqs: List<Faq> = gson.fromJson(faqJson, type)
                assertEquals(faqs.size,2)
                assertEquals(faqs[0].name, "What is 1RM?")
                assertEquals(faqs[0].sort, 1)
                assertEquals(faqs[0].translations[0].languageCode, "hu")
            } else if (record[0] == TrainingPlan::class.simpleName) {
                val trainingPlanJson: String = record[1]
                val type = object : TypeToken<List<TrainingPlan?>?>() {}.type
                val plans: List<TrainingPlan> = gson.fromJson(trainingPlanJson, type)
                assertEquals(plans.size,57)
                assertEquals(plans[1].name, "Beginner Men’s workout")
                assertEquals(plans[1].internalName, "beginner_man")
                assertEquals(plans[1].free, true)
                assertEquals(plans[1].active, true)
                assertEquals(plans[1].treeId, 37)
                assertEquals(plans[0].details[0].exerciseTypeId, 37)
                assertEquals(plans[0].details[1].weight, -1.0)
            } else if (record[0] == SplitTests::class.simpleName) {
                val splitTestJson: String = record[1]
                val type = object : TypeToken<List<SplitTests?>?>() {}.type
                val tests: List<SplitTests> = gson.fromJson(splitTestJson, type)
                assertEquals(tests.size,3)
                assertEquals(tests[1].name, "registration_skip")
                assertEquals(tests[1].testValue, "Colors.cyan")
                assertEquals(tests[2].name, "sales_text")
                assertEquals(tests[2].testValue, "")
                assertEquals(tests[2].source, "description.name")
            } else if (record[0] == TrainingPlanDay::class.simpleName) {
                val trainingPlanDayJson: String = record[1]
                val type = object : TypeToken<List<TrainingPlanDay?>?>() {}.type
                val days: List<TrainingPlanDay> = gson.fromJson(trainingPlanDayJson, type)
                assertEquals(days.size,33)
                assertEquals(days[0].name, "Mo")
                assertEquals(days[0].translations[0].nameTranslation, "Hé")
            } else if (record[0] == AppText::class.simpleName) {
                val appTextJson: String = record[1]
                val type = object : TypeToken<List<AppText?>?>() {}.type
                val texts: List<AppText> = gson.fromJson(appTextJson, type)
                assertEquals(texts.size, 2)
                assertEquals(texts[0].translations[0].translation, "Done!")
                assertEquals(texts[0].translations[1].translation, "Kész!")
            } else if (record[0] == TrainingProgram::class.simpleName) {
                val trainingProgramJson: String = record[1]
                val type = object : TypeToken<List<TrainingProgram?>?>() {}.type
                val programs: List<TrainingProgram> = gson.fromJson(trainingProgramJson, type)
                assertEquals(programs.size, 2)
                assertEquals(programs[0].name, "TopForm")
                assertEquals(programs[1].name, "WeightLoss")
            }
        }
    }

}