package com.aitrainer.api.test.diet

import com.aitrainer.api.model.diet.DietUserConsumption
import com.aitrainer.api.test.Tokenizer
import com.google.gson.Gson
import org.junit.jupiter.api.BeforeAll
import org.junit.jupiter.api.Test
import org.junit.jupiter.api.TestInstance
import org.junit.jupiter.api.extension.ExtendWith
import org.springframework.beans.factory.annotation.Autowired
import org.springframework.boot.test.autoconfigure.web.servlet.AutoConfigureMockMvc
import org.springframework.boot.test.context.SpringBootTest
import org.springframework.http.MediaType
import org.springframework.test.context.junit.jupiter.SpringExtension
import org.springframework.test.web.servlet.MockMvc
import org.springframework.test.web.servlet.request.MockMvcRequestBuilders
import org.springframework.test.web.servlet.request.MockMvcRequestBuilders.get
import org.springframework.test.web.servlet.result.MockMvcResultMatchers.*


@ExtendWith(SpringExtension::class)
@SpringBootTest
@AutoConfigureMockMvc
@TestInstance(TestInstance.Lifecycle.PER_CLASS)
class DietConsumptionTest {

    @Autowired
    private lateinit var mockMvc: MockMvc

    private val dietUserConsumption = DietUserConsumption(
        id = 3,
        dietUserId = 5,
        mealId = 10,
        name = "Tükörtojás",
        dateConsumption = "2023-02-08 17:00",
        quantity = 120.0,
        quantityUnit = "g",
        cal = 70.0,
        protein = 32.0,
        fat = 11.0,
        ch = 1.0,
        sugar = 0.0,
    )

    private var dietUserConsumption2 = DietUserConsumption(
        id = 4,
        dietUserId = 5,
        mealId = 11,
        name = "Rozs zsömle",
        dateConsumption = "2023-02-08 16:00",
        quantity = 80.0,
        quantityUnit = "g",
        cal = 60.0,
        protein = 11.0,
        fat = 4.0,
        ch = 34.0,
        sugar = 8.0,
    )

    private var authToken: String? = ""

    @BeforeAll
    fun initTest() {
        authToken = Tokenizer.getToken()
    }


    @Test
    fun `insert dietUserConsumption successfully`() {

        mockMvc.perform(
            MockMvcRequestBuilders.post("/api/diet_user_consumption")
                .contentType(MediaType.APPLICATION_JSON)
                .header("Authorization", "Bearer $authToken")
                .content(toJson(dietUserConsumption))
        )
        .andExpect(status().isOk)
        .andExpect(jsonPath("$.dietUserId").value(5))
        .andExpect(jsonPath("$.mealId").value(10))
        .andExpect(jsonPath("$.protein").value(32.0))

        mockMvc.perform(
            MockMvcRequestBuilders.post("/api/diet_user_consumption")
                .contentType(MediaType.APPLICATION_JSON)
                .header("Authorization", "Bearer $authToken")
                .content(toJson(dietUserConsumption2))
        ).andExpect(status().isOk)


        val dietUserConsumption3 = DietUserConsumption(
            id = 4,
            dietUserId = 5,
            mealId = 11,
            name = "Rozs zsömle",
            dateConsumption = "2023-02-08 16:30",
            quantity = 80.0,
            quantityUnit = "g",
            cal = 65.0, //**
            protein = 11.0,
            fat = 4.0,
            ch = 44.0, //**
            sugar = 8.0,
        )

        mockMvc.perform(
            MockMvcRequestBuilders.post("/api/diet_user_consumption/4")
                .contentType(MediaType.APPLICATION_JSON)
                .header("Authorization", "Bearer $authToken")
                .content(toJson(dietUserConsumption3))
        )
            .andExpect(status().isOk)
            .andExpect(jsonPath("$.dietUserId").value(5))
            .andExpect(jsonPath("$.mealId").value(11))
            .andExpect(jsonPath("$.sugar").value(8.0))
            .andExpect(jsonPath("$.cal").value(65.0))
            .andExpect(jsonPath("$.ch").value(44.0))

        // Act & Assert
        mockMvc.perform(get("/api/diet_user_consumption/5")
            .header("Authorization", "Bearer $authToken")
            .contentType(MediaType.APPLICATION_JSON))
            .andExpect(status().isOk)
            .andExpect(jsonPath("$.[0].name").value("Tükörtojás"))
            .andExpect(jsonPath("$.[1].name").value("Rozs zsömle"))
            .andExpect(jsonPath("$.[1].ch").value(44))

    }

    private fun toJson(obj: Any): String {
        return Gson().toJson(obj)
    }
}
