package com.aitrainer.api.test.diet

import com.aitrainer.api.model.diet.Diet
import com.aitrainer.api.model.diet.DietMeal
import com.aitrainer.api.test.Tokenizer
import com.google.gson.Gson
import org.junit.jupiter.api.BeforeAll
import org.junit.jupiter.api.Test
import org.junit.jupiter.api.TestInstance
import org.junit.jupiter.api.extension.ExtendWith
import org.springframework.beans.factory.annotation.Autowired
import org.springframework.boot.test.autoconfigure.web.servlet.AutoConfigureMockMvc
import org.springframework.boot.test.context.SpringBootTest
import org.springframework.http.MediaType
import org.springframework.test.context.junit.jupiter.SpringExtension
import org.springframework.test.web.servlet.MockMvc
import org.springframework.test.web.servlet.MvcResult
import org.springframework.test.web.servlet.request.MockMvcRequestBuilders
import org.springframework.test.web.servlet.request.MockMvcRequestBuilders.get
import org.springframework.test.web.servlet.result.MockMvcResultMatchers.*


@ExtendWith(SpringExtension::class)
@SpringBootTest
@AutoConfigureMockMvc
@TestInstance(TestInstance.Lifecycle.PER_CLASS)
class DietTest {

    @Autowired
    private lateinit var mockMvc: MockMvc


    private val diet = Diet(
        dietUserId = 1,
        dietText = "Test diet text",
        startDate = "2023-02-02"
    )

    private var authToken: String? = ""

    @BeforeAll
    fun initTest() {
        authToken = Tokenizer.getToken()
    }


    @Test
    fun `insert diet successfully`() {

        val mvcResult: MvcResult = mockMvc.perform(
            MockMvcRequestBuilders.post("/api/diet")
                .contentType(MediaType.APPLICATION_JSON)
                .header("Authorization", "Bearer $authToken")
                .content(toJson(diet))
        )
        .andExpect(status().isOk)
        .andExpect(jsonPath("$.dietUserId").value(1))
            .andReturn()

        val gson= Gson()
        val newDietJson = mvcResult.response.contentAsString
        val newDiet = gson.fromJson(newDietJson, Diet::class.java)

        mockMvc.perform(get("/api/diet/{dietUserId}", 1)
            .header("Authorization", "Bearer $authToken")
            .contentType(MediaType.APPLICATION_JSON))
            .andExpect(status().isOk)
            .andExpect(jsonPath("$.[0].dietText").value("Test diet text"))
            .andExpect(jsonPath("$.[0].startDate").value("2023-02-02"))

        val meal = DietMeal(
            mealName = "monday|breakfast",
            meal = "Főtt tojás",
            quantity = 2.0,
            mealId = 0,
            mealDate = "2023-03-07 08:00:00"
        )
        val meal2 = DietMeal(
            mealName = "monday|lunch",
            meal = "Disznópörkölt",
            quantity = 120.0,
            mealId = 0,
            mealDate = "2023-03-07 12:00:00"
        )
        val meal3 = DietMeal(
            mealName = "monday|lunch",
            meal = "Savanyúkáposzta",
            quantity = 2.0,
            quantityUnit = "tányér",
            mealId = 3,
            mealDate = "2023-03-07 12:00:00"
        )

        newDiet.meals.add(meal)
        newDiet.meals.add(meal2)
        newDiet.meals.add(meal3)

        //update
        mockMvc.perform(
            MockMvcRequestBuilders.post("/api/diet/${newDiet.dietId}")
                .contentType(MediaType.APPLICATION_JSON)
                .header("Authorization", "Bearer $authToken")
                .content(toJson(newDiet))
        )
            .andExpect(status().isOk)
            .andExpect(jsonPath("$.dietUserId").value(1))
            .andExpect(jsonPath("$.meals[0].meal").value("Főtt tojás"))
            .andExpect(jsonPath("$.meals[0].quantity").value(2.0))
            .andExpect(jsonPath("$.meals[0].mealDate").value("2023-03-07 08:00:00"))
            .andExpect(jsonPath("$.meals[1].mealName").value("monday|lunch"))
            .andExpect(jsonPath("$.meals[2].meal").value("Savanyúkáposzta"))
            .andExpect(jsonPath("$.meals[2].quantity").value(2.0))
            .andExpect(jsonPath("$.meals[2].mealId").value(3))
            .andExpect(jsonPath("$.meals[2].quantityUnit").value("tányér"))

        // update no_text

        newDiet.startDate = "2023-03-20"
        newDiet.meals[0].quantity = 102.0
        mockMvc.perform(
            MockMvcRequestBuilders.post("/api/diet/no_text/${newDiet.dietId}")
                .contentType(MediaType.APPLICATION_JSON)
                .header("Authorization", "Bearer $authToken")
                .content(toJson(newDiet))
        )
            .andExpect(status().isOk)
            .andExpect(jsonPath("$.startDate").value("2023-03-20"))
            .andExpect(jsonPath("$.meals[0].quantity").value(102.0))
            .andExpect(jsonPath("$.dietText").value("Test diet text"))


    }

    private fun toJson(obj: Any): String {
        return Gson().toJson(obj)
    }
}
